console.log('AFL SuperCoach Highlighter: Starting');

const BYE_WEEKS = {
  'SYD': [3, 14],  // Sydney
  'BRL': [3, 16],  // Brisbane Lions
  'CAR': [12],     // Carlton
  'COL': [3, 14],  // Collingwood
  'ESS': [4, 16],  // Essendon
  'FRE': [13],     // Fremantle
  'GEE': [3, 16],  // Geelong
  'GCS': [2, 14],  // Gold Coast
  'GWS': [2, 16],  // GWS Giants
  'HAW': [4, 15],  // Hawthorn
  'MEL': [15],     // Melbourne
  'NTH': [12],     // North Melbourne
  'PTA': [12],     // Port Adelaide
  'RIC': [14],     // Richmond
  'STK': [13],     // St Kilda
  'ADE': [15],     // Adelaide
  'WCE': [15],     // West Coast
  'WBD': [12]      // Western Bulldogs
};

const TEAM_NAME_TO_CODE = {
  'Sydney': 'SYD',
  'Brisbane Lions': 'BRL',
  'Carlton': 'CAR',
  'Collingwood': 'COL',
  'Essendon': 'ESS',
  'Fremantle': 'FRE',
  'Geelong': 'GEE',
  'Gold Coast': 'GCS',
  'GWS Giants': 'GWS',
  'Hawthorn': 'HAW',
  'Melbourne': 'MEL',
  'North Melbourne': 'NTH',
  'Port Adelaide': 'PTA',
  'Richmond': 'RIC',
  'St Kilda': 'STK',
  'Adelaide': 'ADE',
  'West Coast': 'WCE',
  'Western Bulldogs': 'WBD'
};

const COLORS = {
  2: '#FF0000',   // Red
  3: '#0066FF',   // Bright Blue
  4: '#00CC00',   // Bright Green
  12: '#FF1493',  // Deep Pink
  13: '#FF8C00',  // Dark Orange
  14: '#8B008B',  // Dark Magenta
  15: '#008B8B',  // Dark Cyan
  16: '#DAA520'   // Goldenrod
};

let selectedRounds = new Set();

function getTeamCode(element) {
  let teamIdentifier = '';
  
  if (element.title) {
    teamIdentifier = element.title;
  } else if (element.textContent) {
    teamIdentifier = element.textContent.trim();
  }
  
  return TEAM_NAME_TO_CODE[teamIdentifier] || teamIdentifier;
}

function countPlayersForRound(round) {
  const teamElements = document.querySelectorAll('.vm-FieldCellComponent-teamAbbrev span');
  let count = 0;
  
  teamElements.forEach(elem => {
    const teamCode = getTeamCode(elem);
    // Cache the BYE_WEEKS lookup
    const teamByes = BYE_WEEKS[teamCode];
    if (teamCode && teamByes && teamByes.includes(round)) {
      count++;
    }
  });
  
  return count;
}

function updatePlayerCounts() {
  document.querySelectorAll('.bye-counter').forEach((counter, index) => {
    const round = [2, 3, 4, 12, 13, 14, 15, 16][index];
    const playersWithBye = countPlayersForRound(round);
    counter.textContent = `(${playersWithBye} in squad)`;
  });
}

function createUI() {
  console.log('Creating UI');
  const div = document.createElement('div');
  div.className = 'bye-selector';
  
  let html = `
    <div class="header">
      <h3>Bye Round Selector</h3>
      <button type="button" class="toggle-btn" aria-label="Toggle panel">−</button>
    </div>
    <div class="selector-content">
  `;
  
  [2, 3, 4, 12, 13, 14, 15, 16].forEach(round => {
    const playersWithBye = countPlayersForRound(round);
    
    html += `
      <label style="display: flex; justify-content: space-between; align-items: center;">
        <div>
          <input type="checkbox" value="${round}">
          Round ${round}
          <span class="color-sample" style="background-color: ${COLORS[round]}"></span>
        </div>
        <span class="bye-counter" style="margin-left: 10px; color: #666;">
          (${playersWithBye} in squad)
        </span>
      </label>
    `;
  });
  
  html += `
      <button id="clearAll" style="margin-top: 10px; padding: 5px 10px;">Clear All</button>
    </div>
  `;
  
  div.innerHTML = html;
  document.body.appendChild(div);
  
  // Add minimize/maximize functionality
  const toggleBtn = div.querySelector('.toggle-btn');
  let isMinimized = false;
  
  function toggleMinimize() {
    isMinimized = !isMinimized;
    div.classList.toggle('minimized', isMinimized);
    toggleBtn.textContent = isMinimized ? '+' : '−';
  }
  
  toggleBtn.addEventListener('click', toggleMinimize);
  
  // Add event listeners for checkboxes
  const checkboxes = div.querySelectorAll('input[type="checkbox"]');
  
  checkboxes.forEach(checkbox => {
    checkbox.addEventListener('change', (e) => {
      const round = parseInt(e.target.value);
      if (e.target.checked) {
        selectedRounds.add(round);
      } else {
        selectedRounds.delete(round);
      }
      highlightPlayers();
    });
  });
  
  div.querySelector('#clearAll').addEventListener('click', clearHighlights);
  
  // Set up observer for squad changes with debounce
  let updateTimeout;
  const observer = new MutationObserver((mutations) => {
    // Clear any pending timeout
    if (updateTimeout) {
      clearTimeout(updateTimeout);
    }

    // Set new timeout to debounce updates
    updateTimeout = setTimeout(() => {
      let shouldUpdate = false;
      mutations.forEach(mutation => {
        if (mutation.type === 'childList' || 
            (mutation.type === 'attributes' && mutation.target.classList.contains('vm-FieldCellComponent-main'))) {
          shouldUpdate = true;
        }
      });

      if (shouldUpdate) {
        // Clear all highlights first
        document.querySelectorAll('.vm-FieldCellComponent-main').forEach(cell => {
          [2, 3, 4, 12, 13, 14, 15, 16].forEach(round => {
            cell.classList.remove(`bye-week-${round}`);
          });
          cell.title = '';
        });
        
        // Reapply highlights and update counts
        highlightPlayers();
        updatePlayerCounts();
      }
    }, 250); // 250ms debounce
  });

  // Find main container to observe
  const fieldContainer = document.querySelector('.field-container') || document.body;
  
  observer.observe(fieldContainer, {
    childList: true,
    subtree: true,
    attributes: true,
    attributeFilter: ['class']
  });
}

function highlightPlayers() {
  if (selectedRounds.size === 0) {
    clearHighlights();
    return;
  }
  
  const teamElements = document.querySelectorAll('.vm-FieldCellComponent-teamAbbrev span');
  
  // Create a Set of cells to update to avoid duplicates
  const cellsToUpdate = new Set();
  
  teamElements.forEach(elem => {
    const teamCode = getTeamCode(elem);
    const teamByes = BYE_WEEKS[teamCode];
    
    if (!teamCode || !teamByes) return;
    
    const relevantByes = teamByes.filter(round => selectedRounds.has(round));
    
    if (relevantByes.length > 0) {
      let cell = elem.closest('.vm-FieldCellComponent-main');
      if (!cell) {
        cell = elem.closest('[role="cell"]') || 
               elem.closest('.vm-FieldCellComponent') ||
               elem.parentElement?.parentElement?.parentElement;
      }
      
      if (cell) {
        cellsToUpdate.add({cell, teamCode, byes: relevantByes});
      }
    }
  });
  
  // Update all cells at once
  cellsToUpdate.forEach(({cell, teamCode, byes}) => {
    byes.forEach(round => {
      cell.classList.add(`bye-week-${round}`);
    });
    cell.title = `Bye Round${byes.length > 1 ? 's' : ''}: ${byes.join(', ')}`;
  });
}

function clearHighlights() {
  document.querySelectorAll('.vm-FieldCellComponent-main').forEach(cell => {
    [2, 3, 4, 12, 13, 14, 15, 16].forEach(round => {
      cell.classList.remove(`bye-week-${round}`);
    });
    cell.title = '';
  });
  
  document.querySelectorAll('.bye-selector input[type="checkbox"]').forEach(cb => {
    cb.checked = false;
  });
  selectedRounds.clear();
}

function initializeWhenReady(retryCount = 0) {
  const hasPlayers = document.querySelector('.vm-FieldCellComponent-teamAbbrev');
  
  if (!hasPlayers && retryCount < 10) {
    setTimeout(() => initializeWhenReady(retryCount + 1), 1000);
    return;
  }
  
  if (!hasPlayers) {
    return;
  }
  
  createUI();
}

// Start initialization
initializeWhenReady();